!  omegalib.nw --
!
!  Copyright (C) 1999-2024 by
!      Wolfgang Kilian <kilian@physik.uni-siegen.de>
!      Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
!      Juergen Reuter <juergen.reuter@desy.de>
!      with contributions from
!      Fabian Bach <fabian.bach@t-online.de>
!      Bijan Chokoufe Nejad <bijan.chokoufe@desy.de>
!      Christian Speckner <cnspeckn@googlemail.com>     
!
!  WHIZARD is free software; you can redistribute it and/or modify it
!  under the terms of the GNU General Public License as published by
!  the Free Software Foundation; either version 2, or (at your option)
!  any later version.
!
!  WHIZARD is distributed in the hope that it will be useful, but
!  WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with this program; if not, write to the Free Software
!  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
module omega_api_v3
  use kinds
  use omega_birdtracks
  implicit none
  private
  public :: amplitude
  type :: amplitude
     complex(kind=default), dimension(:,:,:,:), allocatable :: amp
     character(:), dimension(:), allocatable :: coupling_orders
     integer, dimension(:,:), allocatable :: coupling_powers
     integer, dimension(:,:), allocatable :: spin_states
     integer, dimension(:,:), allocatable :: flavor_states
     integer :: n_incoming
     type(color_flow), dimension(:), allocatable :: color_flows
  contains
     ! procedure :: copy => copy_amplitude
  end type amplitude
  public :: amplitude_pointer
  type :: amplitude_pointer
     complex(kind=default), dimension(:,:,:,:), pointer :: amp
     character(:), dimension(:), pointer :: coupling_orders
     integer, dimension(:,:), pointer :: coupling_powers
     integer, dimension(:,:), pointer :: spin_states
     integer, dimension(:,:), pointer :: flavor_states
     integer :: n_incoming
     type(color_flow), dimension(:), allocatable :: color_flows
  end type amplitude_pointer
  public :: copy_amplitude
contains
  pure subroutine copy_color_state &
         (cs, i_prt, i_cf, rank_inflowing, inflowing, rank_outflowing, outflowing, is_ghost)
     type(color_state), intent(inout) :: cs
     integer, intent(in) :: i_prt, i_cf
     integer, dimension(:,:), intent(in) :: rank_inflowing, rank_outflowing
     integer, dimension(:,:,:), intent(in) :: inflowing, outflowing
     logical, dimension(:,:), intent(in) :: is_ghost
     cs%g = is_ghost(i_prt,i_cf)
     cs%i = inflowing(1:rank_inflowing(i_prt,i_cf),i_prt,i_cf)
     cs%o = outflowing(1:rank_outflowing(i_prt,i_cf),i_prt,i_cf)
  end subroutine copy_color_state
  pure subroutine copy_color_flow &
         (cf, n_cf, n_incoming, rank_inflowing, inflowing, rank_outflowing, outflowing, is_ghost, &
          n_eps, eps, n_eps_bar, eps_bar)
     type(color_flow), intent(inout) :: cf
     integer, intent(in) :: n_cf, n_incoming
     integer, dimension(:,:), intent(in) :: rank_inflowing, rank_outflowing
     integer, dimension(:), intent(in) :: n_eps, n_eps_bar
     integer, dimension(:,:,:), intent(in) :: inflowing, outflowing, eps, eps_bar
     logical, dimension(:,:), intent(in) :: is_ghost
     integer :: n, n_particles
     n_particles = size (inflowing, dim=2)
     if (.not. allocated (cf%cs)) then
        allocate (cf%cs(n_particles))
     else if (size (cf%cs) /= n_particles) then
        deallocate (cf%cs)
        allocate (cf%cs(n_particles))
     end if
     do n = 1, n_particles
        call copy_color_state (cf%cs(n), n, n_cf, &
          rank_inflowing, inflowing, rank_outflowing, outflowing, is_ghost)
     end do
     if (.not. allocated (cf%e)) then
        allocate (cf%e(n_eps(n_cf)+n_eps_bar(n_cf)))
     else if (size (cf%e) /= n_eps(n_cf) + n_eps_bar(n_cf)) then
        deallocate (cf%e)
        allocate (cf%e(n_eps(n_cf)+n_eps_bar(n_cf)))
     end if
     do n = 1, n_eps(n_cf)
        cf%e(n)%j = eps(:,n,n_cf)
     end do
     do n = 1, n_eps_bar(n_cf)
        cf%e(n+n_eps(n_cf))%j = eps_bar(:,n,n_cf)
     end do
  end subroutine copy_color_flow
  pure subroutine copy_color_flows &
         (cf, n_incoming, rank_inflowing, inflowing, rank_outflowing, outflowing, is_ghost, &
          n_eps, eps, n_eps_bar, eps_bar)
     type(color_flow), dimension(:), allocatable, intent(inout) :: cf
     integer, intent(in) :: n_incoming
     integer, dimension(:,:), intent(in) :: rank_inflowing, rank_outflowing
     integer, dimension(:), intent(in) :: n_eps, n_eps_bar
     integer, dimension(:,:,:), intent(in) :: inflowing, outflowing, eps, eps_bar
     logical, dimension(:,:), intent(in) :: is_ghost
     integer :: n, n_colorflows
     n_colorflows = size (inflowing, dim = 3)
     if (.not. allocated (cf)) then
        allocate (cf(n_colorflows))
     else if (size (cf) /= n_colorflows) then
        deallocate (cf)
        allocate (cf(n_colorflows))
     end if
     do n = 1, n_colorflows
        call copy_color_flow &
           (cf(n), n, n_incoming, rank_inflowing, inflowing, rank_outflowing, outflowing, &
            is_ghost, n_eps, eps, n_eps_bar, eps_bar)
     end do
  end subroutine copy_color_flows
  pure subroutine copy_amplitude &
        (a, n_incoming, flavor_states, spin_states, &
         rank_inflowing, inflowing, rank_outflowing, outflowing, is_ghost, &
         n_eps, eps, n_eps_bar, eps_bar, &
         coupling_orders, coupling_powers, amp)
     type(amplitude), intent(inout) :: a
     integer, intent(in) :: n_incoming
     integer, dimension(:,:), intent(in) :: flavor_states, spin_states
     integer, dimension(:,:), intent(in) :: rank_inflowing, rank_outflowing
     integer, dimension(:), intent(in) :: n_eps, n_eps_bar
     integer, dimension(:,:,:), intent(in) :: inflowing, outflowing, eps, eps_bar
     logical, dimension(:,:), intent(in) :: is_ghost
     character(*), dimension(:), intent(in) :: coupling_orders
     integer, dimension(:,:), intent(in) :: coupling_powers
     complex(kind=default), dimension(:,:,:,:), intent(in) :: amp
     a%amp = amp
     a%coupling_orders = coupling_orders
     a%coupling_powers = coupling_powers
     a%spin_states = spin_states
     a%flavor_states = flavor_states
     a%n_incoming = n_incoming
     call copy_color_flows &
        (a%color_flows, n_incoming, rank_inflowing, inflowing, rank_outflowing, outflowing, &
         is_ghost, n_eps, eps, n_eps_bar, eps_bar)
  end subroutine copy_amplitude
  subroutine init_amplitude_pointer &
        (a, n_incoming, flavor_states, spin_states, &
         rank_inflowing, inflowing, rank_outflowing, outflowing, is_ghost, &
         n_eps, eps, n_eps_bar, eps_bar, &
         coupling_orders, coupling_powers, amp)
     type(amplitude_pointer), intent(inout) :: a
     integer, intent(in) :: n_incoming
     integer, dimension(:,:), target :: flavor_states, spin_states
     integer, dimension(:,:), intent(in) :: rank_inflowing, rank_outflowing
     integer, dimension(:), intent(in) :: n_eps, n_eps_bar
     integer, dimension(:,:,:), intent(in) :: inflowing, outflowing, eps, eps_bar
     logical, dimension(:,:), intent(in) :: is_ghost
     character(*), dimension(:), target :: coupling_orders
     integer, dimension(:,:), target :: coupling_powers
     complex(kind=default), dimension(:,:,:,:), target :: amp
     a%amp => amp
     a%coupling_orders => coupling_orders
     a%coupling_powers => coupling_powers
     a%spin_states => spin_states
     a%flavor_states => flavor_states
     a%n_incoming = n_incoming
     call copy_color_flows &
           (a%color_flows, n_incoming, rank_inflowing, inflowing, rank_outflowing, outflowing, &
            is_ghost, n_eps, eps, n_eps_bar, eps_bar)
  end subroutine init_amplitude_pointer
end module omega_api_v3
