! WHIZARD 3.1.5 Sep 13 2024
!
! Copyright (C) 1999-2024 by
!     Wolfgang Kilian <kilian@physik.uni-siegen.de>
!     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
!     Juergen Reuter <juergen.reuter@desy.de>
!
!     with contributions from
!     cf. main AUTHORS file
!
! WHIZARD is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2, or (at your option)
! any later version.
!
! WHIZARD is distributed in the hope that it will be useful, but
! WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! This file has been stripped of most comments.  For documentation, refer
! to the source 'whizard.nw'
module kinematics_vars

  use kinds
  use lorentz
  use format_defs, only: FMT_19
  implicit none
  private

  public :: mt2_t
  public :: mt2_init
  public :: write_mt2_input
  public :: write_mt2_output
  public :: mt2calc
  public :: mt2_setevent

  ! Calculation parameters
  real(kind=default), parameter, public :: relative_precision = 0.00001, &
       absolute_precision=0.0, min_mass=0.1, step=0.1
  real(kind=default), public :: prec
  ! scale and precision
  ! Define precision=relative_precision*scale where scale=max{Ea, Eb}
  ! Don't set relative precision <0.00001
  ! If mass a < min_mass and mass b < min_mass use massless code
  ! Gives massless particles a very small mass
  ! scanning step size


  type :: mt2_t
     private
     real(default) :: mt2 = 0._default
     real(default) :: mn = 0._default
     real(default) :: ma = 0._default
     real(default) :: mb = 0._default
     real(default) :: Ea2 = 0._default
     real(default) :: Eb2 = 0._default
     real(default) :: pax = 0._default
     real(default) :: pay = 0._default
     real(default) :: pbx = 0._default
     real(default) :: pby = 0._default
     real(default) :: pmx = 0._default
     real(default) :: pmy = 0._default
     real(default) :: scale = 0._default
  end type mt2_t
 

  interface
    module subroutine mt2_init (mt2, mn, ma, mb, Easq, Ebsq, &
         pax, pay, pbx, pby, pmissx, pmissy, scale)
      type(mt2_t), intent(out) :: mt2
      real(default), intent(in) :: mn, ma, mb
      real(default), intent(in) :: Easq, Ebsq
      real(default), intent(in) :: pax, pay, pbx, pby
      real(default), intent(in) :: pmissx, pmissy, scale
    end subroutine mt2_init
    module subroutine write_mt2_input (mt2, u)
      type(mt2_t), intent(in) :: mt2
      integer, intent(in) :: u
    end subroutine write_mt2_input
    module subroutine write_mt2_output (mt2, scale, u)
      real(default), intent(in) :: mt2, scale
      integer, intent(in) :: u
    end subroutine write_mt2_output
    module subroutine mt2calc (mt2, mn, mt2_b)
      type(mt2_t), intent(inout) :: mt2
      real(default), intent(in) :: mn
      real(default), intent(out) :: mt2_b
    end subroutine mt2calc
    module subroutine mt2_setevent (mn, ma, mb, Easq, Ebsq, pax, pay, pbx, pby, &
         pmissx, pmissy, scale, u)
      real(default), intent(out) :: ma, mb ! mass of particles a and b
      real(default), intent(out) :: mn ! invisible particle mass scaled and unscaled  
      real(default), intent(out) :: scale
      real(default), intent(out) :: pax, pay, pbx, pby
      real(default), intent(out) :: pmissx, pmissy
      real(default), intent(out) :: Easq, Ebsq
      integer, intent(in) :: u
    end subroutine mt2_setevent
  end interface

end module kinematics_vars
